@@ -9,26 +9,27 @@ from typing import List, Optional
 
 import ROOT
 
+import quickstats
 from quickstats.components.numerics import is_integer
 
 class ExtendedModel(object):
-    def __init__(self, model_name:str, fname:str, ws_name:str="combWS",
-                 model_config_name:str="ModelConfig", data_name:str="combData", snapshot_names:List[str]=["nominalNuis"],
+    def __init__(self, fname:str, ws_name:str="combWS",
+                 model_config_name:str="ModelConfig", data_name:str="combData", snapshot_name:str="nominalNuis",
                  binned_likelihood:bool=True, tag_as_measurement:str="pdf_",
-                 fix_cache:bool=True, fix_multi:bool=True,
-                 interpolation_code:int=-1):
+                 fix_cache:bool=True, fix_multi:bool=True, interpolation_code:int=-1,
+                 load_extension:bool=True):
         self.fname = fname
-        self.name = model_name
         self.ws_name = ws_name
         self.model_config_name = model_config_name
         self.data_name = data_name
-        self.snapshot_names = snapshot_names
+        self.snapshot_name = snapshot_name
         self.binned_likelihood = binned_likelihood
         self.tag_as_measurement = tag_as_measurement
         self.fix_cache = fix_cache
         self.fix_multi = fix_multi
         self.interpolation_code = interpolation_code
-        
+        if load_extension:
+            self.load_extension()
         self.initialize()
   
     @property
@@ -58,7 +59,16 @@ class ExtendedModel(object):
     @property
     def observables(self):
         return self._observables        
-        
+    
+    @staticmethod
+    def load_extension():
+        try:
+            if not hasattr(ROOT, 'RooTwoSidedCBShape'):
+                ROOT.gSystem.Load(os.path.join(quickstats.macro_path, 'macros', 'RooTwoSidedCBShape_cxx'))
+                if hasattr(ROOT, 'RooTwoSidedCBShape'):
+                    print('INFO: Loaded extension module "RooTwoSidedCBShape"')
+        except Exception:
+            print('WARNING: Failed to load extension module "RooTwoSidedCBShape"')
     @staticmethod
     def modify_interp_codes(ws, interp_code, classes=None):
         if classes is None:
@@ -201,11 +211,9 @@ class ExtendedModel(object):
         self._observables         = observables
                           
         # Load snapshots
-        if self.snapshot_names:
-            for snapshot_name in self.snapshot_names:
-                self._workspace.loadSnapshot(snapshot_name)
-                print('INFO: Loaded snapshot "{}"'.format(snapshot_name))
-        
+        if (self.snapshot_name) and self.workspace.getSnapshot(self.snapshot_name):
+            self._workspace.loadSnapshot(self.snapshot_name)
+            print('INFO: Loaded snapshot "{}"'.format(self.snapshot_name))
         return None
                 
     @staticmethod
@@ -522,7 +530,15 @@ class ExtendedModel(object):
 
     @staticmethod       
     def get_parameters_of_interest(fname:str, ws_name:str="combWS", model_config_name:str="ModelConfig", as_str=True):
-        model_config = ExtendedModel.get_model_config(fname, ws_name, model_config_name)
+        if not os.path.exists(fname):
+            raise FileNotFoundError('workspace file {} does not exist'.format(fname))
+        file = ROOT.TFile(fname) 
+        if (not file):
+            raise RuntimeError("Something went wrong while loading the root file: {}".format(fname))
+        ws = file.Get(ws_name)
+        if not ws:
+            raise RuntimeError("Something went wrong while loading the workspace: {}".format(ws_name))        
+        model_config = ws.obj(model_config_name)
         pois = model_config.GetParametersOfInterest()
         if as_str:
             return [poi.GetName() for poi in pois]                
@@ -531,6 +547,7 @@ class ExtendedModel(object):
 
     @staticmethod
     def get_datasets(fname:str, ws_name:str="combWS", as_str=True):
+        
         ws = ExtendedModel.get_workspace(fname, ws_name)
         if as_str:
             return [data.GetName() for data in ws.allData()]                
